#property copyright "Copyright 2022, Anil Varma"
#property link      "AnilHVarma@gmail.com"
#property version   "1.00"

// ENUM FOR APPLIED PRICES TYPES
enum CENUM_APPLIED_PRICE {
  price_CLOSE,
  price_OPEN,
  price_HIGH,
  price_LOW,
  price_MEDIAN,
  price_TYPICAL,
  price_WEIGHTED,
  price_AVERAGE,
  priceHA_CLOSE,
  priceHA_OPEN,
  priceHA_HIGH,
  priceHA_LOW,
  priceHA_MEDIAN,
  priceHA_TYPICAL,
  priceHA_WEIGHTED,
  priceHA_AVERAGE
};

#include "../MQL5 Forum/EnumsMQL5.mqh"
//+-----------------------------------------------------------------------------------------------------------------------------+
//| CLASS:				CIndicatorBaseMQL5.mqh
//| APPLICATION:	BASE CLASS FOR CUSTOM AND TECHNICAL INDICATORS
//+-----------------------------------------------------------------------------------------------------------------------------+
class CIndicatorBaseMQL5 {

private:

	// MEMBER VARIABLE TO HOLD HEIKEN ASHI PRICE
	double								mHAOpen;
	double								mHAHigh;
	double								mHALow;
	double								mHAClose;
	double 								PriceHA[][4];

	// METHODS TO GET MQL STANDARD PRICE
	double								GetOpen(int pIndex)				{ return(iOpen(mSymbol,mTimeFrame,pIndex));  }
	double								GetHigh(int pIndex)				{ return(iHigh(mSymbol,mTimeFrame,pIndex));  }
	double								GetLow(int pIndex)				{ return(iLow(mSymbol,mTimeFrame,pIndex));   }
	double								GetClose(int pIndex)			{ return(iClose(mSymbol,mTimeFrame,pIndex)); }
	// iVolume() RETURNS TICK VOLUME AS long VARIABLE
	long									GetVolume(int pIndex)			{ return(iVolume(mSymbol,mTimeFrame,pIndex));}

protected:

	// MEMBER VARIABLE TO BE SET BY CHILD CLASS
	string								mSymbol;
	ENUM_TIMEFRAMES				mTimeFrame;
	int										mDigits;

	// MEMBER VARIABLE DEFAULTED IN CInputBase CLASS, HOWEVER CHILD CLASS CAN OVERRIDE VALUES
	int										mShift;														// HORIZONTAL BAR SHIFT
	CENUM_APPLIED_PRICE		mAppliedPrice;
	ENUM_MA_METHOD				mAppliedMAMode;
	ENUM_APPLIED_VOLUME		mAppliedVolume;

	// MEMBER VARIABLE FOR TRACKING IF CHART TIME FRAME OR SYMBOL CHANGED
	datetime							mFirstBarTime;										// TO TRACK, IF NUMBER OF BARS CHANGED ON CHART
	int										mRatesTotal;											// REPLICATE rates_total VARIABLE OF AN INDICATOR
	int										mPrevCalculated;									// REPLICATE prev_calculated VARIABLE OF AN INDICATOR

public:

	CIndicatorBaseMQL5();
 ~CIndicatorBaseMQL5() { }

	void				SetDefaults(CENUM_APPLIED_PRICE pAppliedPrice,ENUM_MA_METHOD pAppliedMAMode);

	// METHOD INTEDENDED TO BE CALLED FROM INDICATORS TO GET APPLIED PRICE AT SPECIFIED INDEX.
	// SYMBOL, TIME FRAME, MQL PRICE ARRAYS AND RATES TOTAL PARAMETERS MUST BE PASSED.
	double 			GetIndexPrice(CENUM_APPLIED_PRICE pAppliedPrice,const double &open[],const double &close[],const double &high[],
                		 				const double &low[],int pIndex,int pRatesTotal);

}; // END Of Class Definition
//+-----------------------------------------------------------------------------------------------------------------------------+
//| METHOD:				CIndicatorBaseMQL5()
//| APPLICATION:	IF CONSTRUCTOR IS USED, DEFAULTS WILL BE SET HERE
//+-----------------------------------------------------------------------------------------------------------------------------+
CIndicatorBaseMQL5::CIndicatorBaseMQL5() {

	mShift 				 = 0;
	mAppliedPrice  = priceHA_CLOSE;
	mAppliedMAMode = MODE_LWMA;
	mAppliedVolume = VOLUME_TICK;														// GetVolume() USES iVolume() WHICH RETURNS ONLY TICK VOLUME

} // END Of method CIndicatorBaseMQL5()
//+-----------------------------------------------------------------------------------------------------------------------------+
//| METHOD:				SetDefaults()
//| APPLICATION:	METHOD TO SET DEFAULTS IN CASE THEY ARE DIFFRENT OR CONSTRUCTOR WAS NOT USED TO SET THEM
//+-----------------------------------------------------------------------------------------------------------------------------+
void CIndicatorBaseMQL5::SetDefaults(CENUM_APPLIED_PRICE pAppliedPrice,ENUM_MA_METHOD pAppliedMAMode) {

	mShift 				 = 0;
	mAppliedPrice  = pAppliedPrice;
	mAppliedMAMode = pAppliedMAMode;
	mAppliedVolume = VOLUME_TICK;														// GetVolume() USES iVolume() WHICH RETURNS ONLY TICK VOLUME

} // END Of method SetDefaults()
//+-----------------------------------------------------------------------------------------------------------------------------+
//| FUNCTION:			GetIndexPrice()
//| APPLICATION:	METHOD TO RETURN SPECIFIED APPLIED PRICE FOR SPECIFIED INDEX
//+-----------------------------------------------------------------------------------------------------------------------------+
double CIndicatorBaseMQL5::GetIndexPrice(CENUM_APPLIED_PRICE pAppliedPrice,const double &open[],const double &close[],
                		 								 const double &high[],const double &low[],int pIndex,int pRatesTotal) {

	int k = pIndex;

	//+---------------------------------------------------------------------------------------------------------------------------+
	//| LOOP TO CALCULATE HEIKEN ASHI CANDLE PRICES
	//+---------------------------------------------------------------------------------------------------------------------------+
		if(pAppliedPrice >= priceHA_CLOSE && pAppliedPrice <= priceHA_AVERAGE) {
	
			// RESET, IF NEEDED PriceHA ARRAY SIZE
	    if(ArrayRange(PriceHA,0) != pRatesTotal)
	    	ArrayResize(PriceHA,pRatesTotal);
	
			// CALCULATE HAOpen PRICE
			double HAOpen;
	    if(k > 0) {
	      HAOpen = (PriceHA[k-1][2] + PriceHA[k-1][3]) / 2.0;
	    }
	    else
	      HAOpen = open[k] + close[k];
	
			// CALCULATE HAClose, HAHigh AND HALow PRICE
	    double HAClose = (open[k] + high[k] + low[k] + close[k]) / 4.0;
	    double HAHigh  = MathMax(high[k],MathMax(HAOpen,HAClose));
	    double HALow   = MathMin(low[k],MathMin(HAOpen,HAClose));
	
			// SET FINAL HA CANDLE PRICES
	    if(HAOpen < HAClose) {
	      PriceHA[k][0] = HALow;
	      PriceHA[k][1] = HAHigh;
	    } 
	    else {
	      PriceHA[k][0] = HAHigh;
	      PriceHA[k][1] = HALow;
	    } 
	    PriceHA[k][2] = HAOpen;
	    PriceHA[k][3] = HAClose;
	
			switch(pAppliedPrice) {
			  case priceHA_CLOSE:     return(HAClose);
			  case priceHA_OPEN:      return(HAOpen);
			  case priceHA_HIGH:      return(HAHigh);
			  case priceHA_LOW:       return(HALow);
			  case priceHA_MEDIAN:    return((HAHigh + HALow) / 2.0);
			  case priceHA_TYPICAL:   return((HAHigh + HALow + HAClose) / 3.0);
			  case priceHA_WEIGHTED:  return((HAHigh + HALow + HAClose + HAClose) / 4.0);
			  case priceHA_AVERAGE:   return((HAHigh + HALow + HAClose + HAOpen)  / 4.0);
			}
		}
	//+---------------------------------------------------------------------------------------------------------------------------+
	//| LOOP TO CALCULATE MQL STANDARD PRICES
	//+---------------------------------------------------------------------------------------------------------------------------+
		else {
			switch(pAppliedPrice) {
				case price_CLOSE:					return(close[k]);
				case price_OPEN:					return(open[k]);
				case price_HIGH:					return(high[k]);
				case price_LOW:						return(low[k]);
				case price_MEDIAN:				return((high[k] + low[k]) / 2.0);
				case price_TYPICAL:				return((high[k] + low[k] + close[k]) / 3.0);
				case price_WEIGHTED:			return((high[k] + low[k] + close[k] + close[k]) / 4.0);
			}
		}

	return(0.00);						// if none of the Price Type above (warning signal!!! to be provided)

} // END Of method GetIndexPrice()
//+-----------------------------------------------------------------------------------------------------------------------------+
